from api_fhir_r4.tests.mixin.genericTestMixin import GenericTestMixin
from api_fhir_r4.tests.mixin.locationTestMixin import LocationTestMixin
from api_fhir_r4.tests.mixin.operationOutcomeTestMixin import OperationOutcomeTestMixin
from api_fhir_r4.tests.mixin.patientTestMixin import PatientTestMixin
from api_fhir_r4.tests.mixin.practitionerTestMixin import PractitionerTestMixin
from api_fhir_r4.tests.mixin.practitionerRoleTestMixin import PractitionerRoleTestMixin
from api_fhir_r4.tests.mixin.claimTestMixin import ClaimTestMixin
from api_fhir_r4.tests.mixin.claimResponseTestMixin import ClaimResponseTestMixin
from api_fhir_r4.tests.mixin.coverageEligibilityRequestTestMixin import CoverageEligibilityRequestTestMixin
from api_fhir_r4.tests.mixin.communicationRequestTestMixin import CommunicationRequestTestMixin
from api_fhir_r4.tests.mixin.genericFhirAPITestMixin import GenericFhirAPITestMixin
from api_fhir_r4.tests.mixin.fhirApiReadTestMixin import FhirApiReadTestMixin
from api_fhir_r4.tests.mixin.fhirApiCreateTestMixin import FhirApiCreateTestMixin
from api_fhir_r4.tests.mixin.fhirApiUpdateTestMixin import FhirApiUpdateTestMixin
from api_fhir_r4.tests.mixin.fhirApiDeleteTestMixin import FhirApiDeleteTestMixin
